##### LIBRARIES #####

library(lavaan)
library(semTools)
library(tidySEM)
library(writexl)
library(broom)
library(dplyr)
library(foreign)
library(utils)
library(forestplot)
library(ggplot2)

##### Sensitivity Analysis (one mediator) - combined & city as binary ####

sensitivity_analysis_c <- function (SEM, indicator_type, indicator_list, mediator, File_Path, bootstraps=1000, estim="DWLS") {
  
  output <- data.frame()
  df <- df_cs3 
  
  for (i in indicator_list) {
    
    colnames(df)[colnames(df) %in% c(mediator) ]<- c("MED")
    colnames(df)[colnames(df) %in% c(i) ]<- c("GS")
    
    SEM001 <- SEM
  
    fit_SEM001 <- sem(data = df, model = SEM001, estimator=estim, se="bootstrap", bootstrap=bootstraps, verbose=F, parallel="yes",
                      ordered = c("MED","WB_c2", "h01", "h01_c"))           # DVs 
    
    parameters <- parameterEstimates(fit_SEM001, boot.ci.type="bca.simple")
    result <- parameters[parameters$label== "GS_MED" | parameters$label== "GS_MED_H" | parameters$label== "direct_GS_H"| parameters$label== "total_GS_H", c("label", "est", "ci.lower", "ci.upper")]
    
    print(result)
    
    output <- rbind(output, result)
    
    cat("Dimensions of output:", dim(output), "\n")
    cat("Dimensions of result:", dim(result), "\n")
    
    colnames(df)[colnames(df) %in% c("MED")] <- c(mediator)
    colnames(df)[colnames(df) %in% c("GS")]  <- c(i)
    
  }
  
  distances <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000, 1100, 1200, 1300, 1400, 1500)
  
  GS_MED      <- cbind(distances, city, output[output$label== "GS_MED",]) 
  GS_MED_H    <- cbind(distances, city, output[output$label== "GS_MED_H",]) 
  GS_H_direct <- cbind(distances, city, output[output$label== "direct_GS_H",]) 
  GS_H_total  <- cbind(distances, city, output[output$label== "total_GS_H",]) 
  
  
  # Save dataframes
  file_path <- File_Path
  
  file_name1 <- paste0 (indicator_type, "_GS_MED.csv")
  write.csv(GS_MED, paste0(file_path, file_name1), row.names = TRUE)
  print(c("Saved as:", paste0(file_path, file_name1)))
  
  file_name2 <- paste0 (indicator_type, "_GS_MED_H.csv")
  write.csv(GS_MED_H, paste0(file_path, file_name2), row.names = TRUE)
  print(c("Saved as:", paste0(file_path, file_name2)))
  
  file_name3 <- paste0 (indicator_type, "_GS_H_direct.csv")
  write.csv(GS_H_direct, paste0(file_path, file_name3), row.names = TRUE)
  print(c("Saved as:", paste0(file_path, file_name3)))
  
  file_name4 <- paste0 (indicator_type, "_GS_H_total.csv")
  write.csv(GS_H_total, paste0(file_path, file_name4), row.names = TRUE)
  print(c("Saved as:", paste0(file_path, file_name4)))
  
  # Sensitivity forestplot
  df_list <- list(GS_MED, GS_MED_H, GS_H_direct, GS_H_total)
  
  for (df in df_list) {
    
    if (identical(df, GS_MED)) {
      df_name <- "GS_MED"
      
    } else if (identical(df, GS_MED_H)) {
      df_name <- "GS_MED_H"
      
    } else if (identical(df, GS_H_direct)) {
      df_name <- "GS_H_direct"
      
    } else {
      df_name <- "GS_H_total"
    }
    
    plot_df <- df #[order(df$distances),]
    plot_df$estimates <- paste0(sprintf("%.3f",round(plot_df$est, digits=3)), " [", sprintf("%.3f",round(plot_df$ci.lower, digits=3)), ", ", sprintf("%.3f",round(plot_df$ci.upper, digits=3)), "]")
    plot_df$row_names <- paste(plot_df$distances, " ", plot_df$city, " ", plot_df$estimates)
    plot_df$row_names
    
    ggplot(plot_df, aes(x = distances, y = est)) +
      ggtitle(df_name) + 
      geom_line(size = 1.5) +
      geom_point(size = 2.5) +
      geom_ribbon(aes(ymin = ci.lower, ymax = ci.upper), alpha = 0.2) +
      labs(x = "distance", y = "ß [95% CI]") +
      theme_minimal() +
      geom_hline(yintercept = 0) +
      scale_x_continuous(breaks = distances) +
      theme(axis.line.x = element_line(size = 1.5), axis.text.x = element_text(size = 24), axis.line.y = element_line(size = 0), axis.text.y = element_text(size = 24))
    
    # Save plot
    file_path <- File_Path
    file_name <- paste0 (indicator_type, "_", df_name, "_forestplot.png")
    ggsave(paste0(file_path, file_name))   
  }  
}
